@@ -33,11 +33,11 @@
 %     'boot1way (..., 'bootfun', BOOTFUN)' also specifies BOOTFUN: the function
 %     calculated on the original sample and the bootstrap resamples. BOOTFUN
 %     must be either a:
-%        o function handle or anonymous function,
-%        o string of function name, or
+%        o function handle, function name or an anonymous function,
+%        o string of a function name, or
 %        o a cell array where the first cell is one of the above function
 %          definitions and the remaining cells are (additional) input arguments 
-%          to that function (other than the data arguments).
+%          to that function (after the data arguments).
 %        In all cases, BOOTFUN must take DATA for the initial input argument(s).
 %        BOOTFUN must calculate a statistic representative of the finite data
 %        sample; it should NOT be an estimate of a population parameter (unless
@@ -67,14 +67,14 @@
 %     and how to perform bootstrap iterations using multiple processors (if the
 %     Parallel Computing Toolbox or Octave Parallel package is available). This
 %     argument is a structure with the following recognised fields:
-%        o 'UseParallel':  If true, use parallel processes to accelerate
-%                          bootstrap computations on multicore machines,
-%                          specifically non-vectorized function evaluations,
-%                          double bootstrap resampling and jackknife function
-%                          evaluations. Default is false for serial computation.
-%                          In MATLAB, the default is true if a parallel pool
-%                          has already been started. 
-%        o 'nproc':        nproc sets the number of parallel processes
+%        o 'UseParallel': If true, use parallel processes to accelerate
+%                         bootstrap computations on multicore machines,
+%                         specifically non-vectorized function evaluations,
+%                         double bootstrap resampling and jackknife function
+%                         evaluations. Default is false for serial computation.
+%                         In MATLAB, the default is true if a parallel pool
+%                         has already been started. 
+%        o 'nproc':       nproc sets the number of parallel processes (optional)
 %
 %     'PVAL = boot1way (DATA, GROUP, ...)' returns the p-value(s) for the 
 %     (multiple) two-tailed test(s). Note that the p-value(s) returned are
@@ -149,8 +149,8 @@ function [pval, c, stats, Q] = boot1way (data, group, varargin)
                    ' atleast 2 input arguments required'))
   end
 
-  % Store local functions in a stucture for parallel processes
-  localfunc = struct ('maxstat',@maxstat, ...
+  % Store subfunctions in a stucture to make them available for parallel processes
+  parsubfun = struct ('maxstat',@maxstat, ...
                       'bootcdf',@bootcdf);
 
   % Check if running in Octave (else assume Matlab)
@@ -175,27 +175,32 @@ function [pval, c, stats, Q] = boot1way (data, group, varargin)
   argin3 = varargin;
   narg = numel (argin3);
   if (narg > 1)
-    while ischar (argin3{end-1})
-      if (strcmpi (argin3{end-1},'bootfun'))
-        bootfun = argin3{end};
-      elseif (strcmpi (argin3{end-1},'nboot'))
-        nboot = argin3{end};
-      elseif (strcmpi (argin3{end-1},'ref'))
-        ref = argin3{end};
-      elseif (any (strcmpi (argin3{end-1},{'Options','Option'})))
-        paropt = argin3{end};
-      elseif (strcmpi (argin3{end-1},'alpha'))
-        alpha = argin3{end};
-      elseif (any (strcmpi (argin3{end-1},{'DisplayOpt','Display'})))
-        DisplayOpt = argin3{end};
-      else
-        error ('boot1way: Unrecognised input argument to boot1way')
+    name = argin3{end - 1};
+    value = argin3{end};
+    while (ischar (name))
+      switch (lower (name))
+        case 'bootfun'
+          bootfun = value;
+        case 'nboot'
+          nboot = value;
+        case 'ref'
+          ref = value;
+        case {'options','option'}
+          paropt = value;
+        case 'alpha'
+          alpha = value;
+        case {'displayopt','display'}
+          DisplayOpt = value;
+        otherwise
+          error ('boot1way: Unrecognised input argument to boot1way')
       end
-      argin3 = {argin3{1:end-2}};
+      argin3 = argin3(1:end-2);
       narg = numel (argin3);
       if (narg < 1)
         break
       end
+      name = argin3{end - 1};
+      value = argin3{end};
     end
   end
 
@@ -395,7 +400,7 @@ function [pval, c, stats, Q] = boot1way (data, group, varargin)
     if (paropt.UseParallel && (paropt.nproc > 1) && ~PARALLEL)
       if (ISOCTAVE)
         % OCTAVE Parallel Computing Package is not installed or loaded
-        warning ('boot1way:parallel', cat (2, 'Parallel Computing Package', ...
+        warning ('boot1way:parallel', cat (2, 'Parallel package', ...
          ' is not installed and/or loaded. Falling back to serial processing.'))
       else
         % MATLAB Parallel Computing Toolbox is not installed or loaded
@@ -408,7 +413,7 @@ function [pval, c, stats, Q] = boot1way (data, group, varargin)
   end
 
   % Create maxstat anonymous function for bootstrap
-  func = @(data) localfunc.maxstat (data, g, nboot(2), bootfun, ref, ISOCTAVE);
+  func = @(data) parsubfun.maxstat (data, g, nboot(2), bootfun, ref, ISOCTAVE);
 
   % Perform resampling and calculate bootstrap statistics to estimate the 
   % sampling distribution under the null hypothesis.
@@ -969,12 +974,12 @@ end
 %! p = boot1way (y(:),g(:),'ref','male','nboot',[999,0],'DisplayOpt',false);
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
-%!   assert (p, 0.281036161800208, 1e-06);
+%!   assert (p, 0.2770321577962039, 1e-06);
 %! end
 %! p = boot1way (y(:),g(:),'nboot',[999,0],'DisplayOpt',false);
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
-%!   assert (p, 0.281036161800208, 1e-06);
+%!   assert (p, 0.2770321577962039, 1e-06);
 %! end
 %! % Result from anova1 is 0.2613
 
@@ -1015,7 +1020,7 @@ end
 %! p = boot1way (y(:),g(:),'bootfun',{@std,1},'DisplayOpt',false);
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
-%!   assert (min(p), 0.4523926257950379, 1e-06);
+%!   assert (min(p), 0.4455958708255711, 1e-06);
 %! end
 
 %!test
