@@ -87,14 +87,14 @@
 %     and how to perform bootstrap iterations using multiple processors (if the
 %     Parallel Computing Toolbox or Octave Parallel package is available). This
 %     argument is a structure with the following recognised fields:
-%       <> 'UseParallel':  If true, use parallel processes to accelerate
-%                          bootstrap computations on multicore machines,
-%                          specifically non-vectorized function evaluations,
-%                          double bootstrap resampling and jackknife function
-%                          evaluations. Default is false for serial computation.
-%                          In MATLAB, the default is true if a parallel pool
-%                          has already been started. 
-%       <> 'nproc':        nproc sets the number of parallel processes
+%       <> 'UseParallel': If true, use parallel processes to accelerate
+%                         bootstrap computations on multicore machines,
+%                         specifically non-vectorized function evaluations,
+%                         double bootstrap resampling and jackknife function
+%                         evaluations. Default is false for serial computation.
+%                         In MATLAB, the default is true if a parallel pool
+%                         has already been started. 
+%       <> 'nproc':       nproc sets the number of parallel processes (optional)
 %
 %     '[CI, BOOTSTAT] = bootci (...)' also returns the bootstrap statistics
 %     used to calculate the confidence intervals CI.
@@ -181,33 +181,38 @@ function [ci, bootstat, bootsam] = bootci (argin1, argin2, varargin)
   argin3 = varargin;
   narg = numel (argin3);
   if (narg > 1)
-    while (ischar (argin3{end-1}))
-      if (any (strcmpi ({'Options', 'Option'}, argin3{end-1})))
-        paropt = argin3{end};
-      elseif (any (strcmpi ('alpha', argin3{end-1})))
-        alpha = argin3{end};
-      elseif (any (strcmpi ('type', argin3{end-1})))
-        type = argin3{end};
-      elseif (any (strcmpi ('nbootstd', argin3{end-1})))
-        nbootstd = argin3{end};
-      elseif (any (strcmpi ('nbootcal', argin3{end-1})))
-        nbootcal = argin3{end};
-      elseif (any (strcmpi ('strata', argin3{end-1})))
-        strata = argin3{end};
-      elseif (any (strcmpi ('loo', argin3{end-1})))
-        loo = argin3{end};
-      elseif (any (strcmpi ('seed', argin3{end-1})))
-        seed = argin3{end};
-        % Initialise the random number generator with the seed
-        boot (1, 1, true, seed);
-      else
-        error ('bootci: Unrecognised input argument to bootci')
+    name = argin3{end - 1};
+    value = argin3{end};
+    while (ischar (name))
+      switch (lower (name))
+        case {'options', 'option'}
+          paropt = value;
+        case 'alpha'
+          alpha = value;
+        case 'type'
+          type = value;
+        case 'nbootstd'
+          nbootstd = value;
+        case 'nbootcal'
+          nbootcal = value;
+        case 'strata'
+          strata = value;
+        case 'loo'
+          loo = value;
+        case'seed'
+          seed = value;
+          % Initialise the random number generator with the seed
+          boot (1, 1, true, seed);
+        otherwise
+          error ('bootci: Unrecognised input argument to bootci')
       end
-      argin3 = {argin3{1:end-2}};
+      argin3 = argin3(1:end-2);
       narg = numel (argin3);
       if (narg < 2)
         break
       end
+      name = argin3{end - 1};
+      value = argin3{end};
     end
   end
   if (iscell (argin2))
@@ -230,7 +235,15 @@ function [ci, bootstat, bootsam] = bootci (argin1, argin2, varargin)
     end
   end
   if (paropt.UseParallel)
-    ncpus = paropt.nproc;
+    if (isfield (paropt, 'nproc'))
+      ncpus = paropt.nproc;
+    else
+      if (ISOCTAVE)
+        ncpus = inf;
+      else
+        ncpus = [];
+      end
+    end
   else
     ncpus = 0;
   end
@@ -289,8 +302,8 @@ function [ci, bootstat, bootsam] = bootci (argin1, argin2, varargin)
       if (ncpus == 0)
         % OCTAVE Parallel Computing Package is not installed or loaded
         warning ('bootci:parallel', ...
-                 cat (2, 'Parallel Computing Package is installed and/or', ...
-                         ' loaded. Falling back to serial processing.'))
+                 cat (2, 'Parallel package is not installed and/or loaded.', ...
+                         ' Falling back to serial processing.'))
       end
     else
       info = ver; 
@@ -408,8 +421,8 @@ function [ci, bootstat, bootsam] = bootci (argin1, argin2, varargin)
 
       % Other interval types are natively supported in bootknife function
       % Undocumented input argument for simple bootstrap resampling
-      [stats, bootstat] = bootknife (data, nboot, bootfun, alpha, strata, ...
-                                     ncpus, [], [], ISOCTAVE, true, loo);
+      [stats, bootstat, bootsam] = bootknife (data, nboot, bootfun, alpha, ...
+                                     strata, ncpus, [], [], ISOCTAVE, true, loo);
 
   end
 
@@ -431,6 +444,8 @@ end
 %!
 %! % 95% BCa bootstrap confidence intervals for the mean
 %! ci = bootci (1999, @mean, data)
+%!
+%! % Please be patient, the calculations will be completed soon...
 
 %!demo
 %!
@@ -463,6 +478,8 @@ end
 %!
 %! % 90% percentile bootstrap confidence intervals for the variance
 %! ci = bootci (1999, {{@var,1}, data}, 'type', 'per', 'alpha', 0.1)
+%!
+%! % Please be patient, the calculations will be completed soon...
 
 %!demo
 %!
@@ -472,6 +489,8 @@ end
 %!
 %! % 90% BCa bootstrap confidence intervals for the variance
 %! ci = bootci (1999, {{@var,1}, data}, 'type', 'bca', 'alpha', 0.1)
+%!
+%! % Please be patient, the calculations will be completed soon...
 
 %!demo
 %!
@@ -606,6 +625,21 @@ end
 %!   %<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 %! 
 %! end
+%!
+%! % Please be patient, the calculations will be completed soon...
+
+%!demo
+%!
+%! % Input dataset
+%! y = randn (20,1); x = randn (20,1); X = [ones(20,1), x];
+%!
+%! % 95% BCa confidence interval for regression coefficients 
+%! ci = bootci (1999, @mldivide, X, y)
+%!
+%! % N.B: Consider using either the 'bootwild', 'bootbayes' or 'bootlm'
+%! % functions for bootstrapping linear regression problems instead
+%!
+%! % Please be patient, the calculations will be completed soon...
 
 %!demo
 %!
@@ -708,12 +742,12 @@ end
 %! %
 %! % method             |   0.05 |   0.95 | length | shape |  
 %! % -------------------|--------|--------|--------|-------|
-%! % ci1 - normal       |  110.1 |  246.2 |  136.1 |  1.22 |
-%! % ci2 - percentile   |   98.1 |  234.7 |  136.6 |  0.86 |
-%! % ci3 - basic        |  108.4 |  245.0 |  136.1 |  1.17 |
-%! % ci4 - BCa          |  116.1 |  259.3 |  143.2 |  1.59 |
-%! % ci5 - bootstrap-t  |  114.0 |  290.3 |  176.3 |  2.07 |
-%! % ci6 - calibrated   |  115.3 |  276.4 |  161.1 |  1.87 |
+%! % ci1 - normal       |  108.9 |  247.4 |  138.5 |  1.22 |
+%! % ci2 - percentile   |   98.2 |  235.1 |  136.9 |  0.87 |
+%! % ci3 - basic        |  108.0 |  244.8 |  136.9 |  1.15 |
+%! % ci4 - BCa          |  116.2 |  259.6 |  143.4 |  1.60 |
+%! % ci5 - bootstrap-t  |  114.0 |  289.5 |  175.5 |  2.05 |
+%! % ci6 - calibrated   |  115.3 |  276.9 |  161.6 |  1.87 |
 %! % -------------------|--------|--------|--------|-------|
 %! % parametric - exact |  118.4 |  305.2 |  186.8 |  2.52 |
 %! %
@@ -780,15 +814,15 @@ end
 %!   y = randn (20,1); x = randn (20,1); X = [ones(20,1),x];
 %!   bootci (1999, @cor, x, y);
 %!   bootci (1999, {@cor, x, y}, 'strata', g);
-%!   bootci (1999, @(y,X) pinv(X)*y, y, X);
-%!   bootci (1999, @(y,X) pinv(X)*y, y, X, 'alpha', 0.1);
-%!   bootci (1999, {@(y,X) pinv(X)*y, y, X}, 'alpha', 0.1);
-%!   bootci (1999, {@(y,X) pinv(X)*y, y, X}, 'alpha', 0.1, 'type', 'norm');
-%!   bootci (1999, {@(y,X) pinv(X)*y, y, X}, 'alpha', 0.1, 'type', 'per');
-%!   bootci (1999, {@(y,X) pinv(X)*y, y, X}, 'alpha', 0.1, 'type', 'basic');
-%!   bootci (1999, {@(y,X) pinv(X)*y, y, X}, 'alpha', 0.1, 'type', 'bca');
-%!   bootci (1999, {@(y,X) pinv(X)*y, y, X}, 'alpha', 0.1, 'type', 'stud');
-%!   bootci (1999, {@(y,X) pinv(X)*y, y, X}, 'alpha', 0.1, 'type', 'cal');
+%!   bootci (1999, @mldivide, X, y);
+%!   bootci (1999, @mldivide, X, y, 'alpha', 0.1);
+%!   bootci (1999, {@mldivide, X, y}, 'alpha', 0.1);
+%!   bootci (1999, {@mldivide, X, y}, 'alpha', 0.1, 'type', 'norm');
+%!   bootci (1999, {@mldivide, X, y}, 'alpha', 0.1, 'type', 'per');
+%!   bootci (1999, {@mldivide, X, y}, 'alpha', 0.1, 'type', 'basic');
+%!   bootci (1999, {@mldivide, X, y}, 'alpha', 0.1, 'type', 'bca');
+%!   bootci (1999, {@mldivide, X, y}, 'alpha', 0.1, 'type', 'stud');
+%!   bootci (1999, {@mldivide, X, y}, 'alpha', 0.1, 'type', 'cal');
 %! catch
 %!   warning ('on', 'bootknife:parallel')
 %!   rethrow (lasterror)
@@ -807,8 +841,8 @@ end
 %! ci = bootci(1999,{{@var,1},A},'alpha',0.1,'type','per','seed',1);
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
-%!   assert (ci(1), 97.67001810580366, 1e-07);
-%!   assert (ci(2), 233.0388437331518, 1e-07);
+%!   assert (ci(1), 97.67455621301775, 1e-07);
+%!   assert (ci(2), 233.1553254437869, 1e-07);
 %! end
 %!
 %! % Nonparametric 90% BCa confidence intervals (single bootstrap)
@@ -816,8 +850,8 @@ end
 %! ci = bootci(1999,{{@var,1},A},'alpha',0.1,'type','bca','seed',1);
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
-%!   assert (ci(1), 115.956346143457, 1e-07);
-%!   assert (ci(2), 256.2289974055464, 1e-07);
+%!   assert (ci(1), 115.3439904214919, 1e-07);
+%!   assert (ci(2), 254.8041855869813, 1e-07);
 %! end
 %!
 %! % Nonparametric 90% bootstrap-t confidence intervals (double bootstrap)
@@ -825,8 +859,8 @@ end
 %!                                                  'nbootstd',100,'seed',1);
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
-%!   assert (ci(1), 114.5922631417029, 1e-07);
-%!   assert (ci(2), 287.0028178891804, 1e-07);
+%!   assert (ci(1), 114.4602660067695, 1e-07);
+%!   assert (ci(2), 288.7667036808156, 1e-07);
 %! end
 %!
 %! % Nonparametric 90% calibrated percentile confidence intervals
@@ -835,8 +869,8 @@ end
 %!                                                  'nbootcal',199,'seed',1);
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
-%!   assert (ci(1), 116.0773028946106, 1e-07);
-%!   assert (ci(2), 279.0402847885312, 1e-07);
+%!   assert (ci(1), 116.1158249047411, 1e-07);
+%!   assert (ci(2), 277.5572125379819, 1e-07);
 %! end
 %!
 %! % Exact intervals based on normal theory are 118.4 - 305.2 (Table 14.2)
@@ -854,8 +888,8 @@ end
 %! ci = bootci(1999,{@cor,LSAT,GPA},'alpha',0.1,'type','per','seed',1);
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
-%!   assert (ci(1), 0.5146251204001586, 1e-07);
-%!   assert (ci(2), 0.9531054945982934, 1e-07);
+%!   assert (ci(1), 0.5087487217143066, 1e-07);
+%!   assert (ci(2), 0.9477093289635946, 1e-07);
 %! end
 %!
 %! % Nonparametric 90% BCa confidence intervals (single bootstrap)
@@ -863,8 +897,8 @@ end
 %! ci = bootci(1999,{@cor,LSAT,GPA},'alpha',0.1,'type','bca','seed',1);
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
-%!   assert (ci(1), 0.4177828971982108, 1e-07);
-%!   assert (ci(2), 0.9238952404759969, 1e-07);
+%!   assert (ci(1), 0.4107777334127511, 1e-07);
+%!   assert (ci(2), 0.9234715078308394, 1e-07);
 %! end
 %!
 %! % Nonparametric 90% calibrated percentile confidence intervals
@@ -873,7 +907,7 @@ end
 %!                                                     'nbootcal',499,'seed',1);
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
-%!   assert (ci(1), 0.2832002889846569, 1e-07);
-%!   assert (ci(2), 0.9352503844648827, 1e-07);
+%!   assert (ci(1), 0.2899032097111454, 1e-07);
+%!   assert (ci(2), 0.9347549589238046, 1e-07);
 %! end
 %! % Exact intervals based on normal theory are 0.51 - 0.91
