@@ -33,11 +33,11 @@
 %     'bootclust (DATA, NBOOT, BOOTFUN)' also specifies BOOTFUN: the function
 %     calculated on the original sample and the bootstrap resamples. BOOTFUN
 %     must be either a:
-%       <> function handle or anonymous function,
-%       <> string of function name, or
+%       <> function handle, function name or an anonymous function,
+%       <> string of a function name, or
 %       <> a cell array where the first cell is one of the above function
 %          definitions and the remaining cells are (additional) input arguments 
-%          to that function (other than the data arguments).
+%          to that function (after the data arguments).
 %        In all cases BOOTFUN must take DATA for the initial input argument(s).
 %        BOOTFUN can return a scalar or any multidimensional numeric variable,
 %        but the output will be reshaped as a column vector. BOOTFUN must
@@ -108,10 +108,6 @@
 %     NBOOT cell array of datasets generated by cluster or block bootstrap
 %     resampling.
 %
-%  REQUIREMENTS:
-%    The function file boot.m (or better boot.mex) and bootcdf, which are
-%    distributed with the statistics-resampling package.
-%
 %  BIBLIOGRAPHY:
 %  [1] Davison and Hinkley (1997). Bootstrap methods and their application
 %        (Vol. 1). New York, NY: Cambridge University Press.
@@ -148,8 +144,8 @@
 function [stats, bootstat, X] = bootclust (x, nboot, bootfun, alpha, ...
                                         clustid, loo, seed, ncpus)
 
-  % Store local functions in a stucture for parallel processes
-  localfunc = struct ('col2args', @col2args, ...
+  % Store subfunctions in a stucture to make them available for parallel processes
+  parsubfun = struct ('col2args', @col2args, ...
                       'kdeinv', @kdeinv, ...
                       'ExpandProbs', @ExpandProbs);
 
@@ -344,8 +340,8 @@ function [stats, bootstat, X] = bootclust (x, nboot, bootfun, alpha, ...
       if (ISOCTAVE)
         % OCTAVE Parallel Computing Package is not installed or loaded
         warning ('bootknife:parallel', ...
-                 cat (2, 'Parallel Computing Package not installed and/or', ...
-                         ' loaded. Falling back to serial processing.'))
+                 cat (2, 'Parallel package is not installed and/or loaded.', ...
+                         ' Falling back to serial processing.'))
       else
         % MATLAB Parallel Computing Toolbox is not installed or loaded
         warning ('bootknife:parallel', ...
@@ -361,7 +357,7 @@ function [stats, bootstat, X] = bootclust (x, nboot, bootfun, alpha, ...
   if (iscell (x))
     szx = cellfun (@(x) size (x, 2), x);
     x = [x{:}];
-    bootfun = @(x) localfunc.col2args (bootfun, x, szx);
+    bootfun = @(x) parsubfun.col2args (bootfun, x, szx);
   else
     szx = size (x, 2);
   end
@@ -501,13 +497,13 @@ function [stats, bootstat, X] = bootclust (x, nboot, bootfun, alpha, ...
   % First, if bootfun is the arithmetic meam, expand the probabilities of the 
   % percentiles for the confidence intervals using Student's t-distribution
   if (strcmpi (bootfun_str, 'mean'))
-    probs = localfunc.ExpandProbs (probs, nx - 1, loo);
+    probs = parsubfun.ExpandProbs (probs, nx - 1, loo);
   end
   % If requested, perform adjustments to the probabilities to correct for bias
   % and skewness
   switch (nalpha)
     case 1
-      % No adjustements made
+      % No adjustments made
       probs = repmat (probs, m, 1);
     case 2
       % Create distribution functions
@@ -521,21 +517,39 @@ function [stats, bootstat, X] = bootclust (x, nboot, bootfun, alpha, ...
         warning ('off', 'all');
       end
       try
-        jackfun = @(i) bootfun (vertcat (x{1 : nx ~= i, :}));
-        if (ncpus > 1)  
-          % PARALLEL evaluation of bootfun on each jackknife resample 
-          if (ISOCTAVE)
-            % OCTAVE
-            T = cell2mat (pararrayfun (ncpus, jackfun, 1 : nx, ...
-                                       'UniformOutput', false));
+        if (VECTORIZED)
+          % Leave-one-out DATA resampling followed by vectorized function
+          % evaluations
+          if (nvar > 1)
+            % Multivariate
+            T = bootfun (reshape (cell2mat (arrayfun (...
+                         @(i) vertcat (x{1 : nx ~= i, :}), (1 : nx)', ...
+                         'UniformOutput', false)), n - n / nx, []));
           else
-            % MATLAB
-            T = zeros (m, nx);
-            parfor i = 1 : nx; T(:, i) = feval (jackfun, i); end
+            % Univariate
+            T = bootfun (cell2mat (arrayfun (...
+                         @(i) vertcat (x{1 : nx ~= i, :}), (1 : nx), ...
+                         'UniformOutput', false)));
           end
         else
-          % SERIAL evaluation of bootfun on each jackknife resample
-          T = cell2mat (arrayfun (jackfun, 1 : nx, 'UniformOutput', false));
+          % Leave-one-out DATA resampling followed by looped function
+          % evaluations (if bootfun is not vectorized)
+          jackfun = @(i) bootfun (vertcat (x{1 : nx ~= i, :}));
+          if (ncpus > 1)  
+            % PARALLEL evaluation of bootfun on each jackknife resample 
+            if (ISOCTAVE)
+              % OCTAVE
+              T = cell2mat (pararrayfun (ncpus, jackfun, 1 : nx, ...
+                                         'UniformOutput', false));
+            else
+              % MATLAB
+              T = zeros (m, nx);
+              parfor i = 1 : nx; T(:, i) = feval (jackfun, i); end
+            end
+          else
+            % SERIAL evaluation of bootfun on each jackknife resample
+            T = cell2mat (arrayfun (jackfun, 1 : nx, 'UniformOutput', false));
+          end
         end
         % Calculate empirical influence function
         U = (nx - 1) * bsxfun (@minus, mean (T, 2), T);
@@ -567,7 +581,7 @@ function [stats, bootstat, X] = bootclust (x, nboot, bootfun, alpha, ...
   ci = nan (m, 2);
   for j = 1 : m
     try
-      ci(j, :) = localfunc.kdeinv (probs(j, :), bootstat(j, :), ...
+      ci(j, :) = parsubfun.kdeinv (probs(j, :), bootstat(j, :), ...
                          se(j) * sqrt (1 / (nx - 1)), 1 - 1 / (nx - 1));
     catch
       % Linear interpolation (legacy)
@@ -800,6 +814,8 @@ end
 %!
 %! % 95% expanded BCa bootstrap confidence intervals for the mean
 %! bootclust (data, 1999, @mean);
+%!
+%! % Please be patient, the calculations will be completed soon...
 
 %!demo
 %!
@@ -812,6 +828,8 @@ end
 %! % 95% expanded BCa bootstrap confidence intervals for the mean with
 %! % cluster resampling
 %! bootclust (data, 1999, @mean, [0.025,0.975], clustid);
+%!
+%! % Please be patient, the calculations will be completed soon...
 
 %!demo
 %!
@@ -822,6 +840,8 @@ end
 %! % 90% equal-tailed percentile bootstrap confidence intervals for
 %! % the variance
 %! bootclust (data, 1999, {@var, 1}, 0.1);
+%!
+%! % Please be patient, the calculations will be completed soon...
 
 %!demo
 %!
@@ -834,6 +854,8 @@ end
 %! % 90% equal-tailed percentile bootstrap confidence intervals for
 %! % the variance
 %! bootclust (data, 1999, {@var, 1}, 0.1, clustid);
+%!
+%! % Please be patient, the calculations will be completed soon...
 
 %!demo
 %!
@@ -843,6 +865,8 @@ end
 %!
 %! % 90% BCa bootstrap confidence intervals for the variance
 %! bootclust (data, 1999, {@var, 1}, [0.05 0.95]);
+%!
+%! % Please be patient, the calculations will be completed soon...
 
 %!demo
 %!
@@ -854,6 +878,8 @@ end
 %!
 %! % 90% BCa bootstrap confidence intervals for the variance
 %! bootclust (data, 1999, {@var, 1}, [0.05 0.95], clustid);
+%!
+%! % Please be patient, the calculations will be completed soon...
 
 %!demo
 %!
@@ -861,7 +887,9 @@ end
 %! y = randn (20,1); x = randn (20,1); X = [ones(20,1), x];
 %!
 %! % 90% BCa confidence interval for regression coefficients 
-%! bootclust ({y,X}, 1999, @(y,X) X\y, [0.05 0.95]); % Could also use @regress
+%! bootclust ({X,y}, 1999, @mldivide, [0.05 0.95]);
+%!
+%! % Please be patient, the calculations will be completed soon...
 
 %!demo
 %!
@@ -870,7 +898,9 @@ end
 %! clustid = [1;1;1;1;2;2;2;3;3;3;3;4;4;4;4;4;5;5;5;6];
 %!
 %! % 90% BCa confidence interval for regression coefficients 
-%! bootclust ({y,X}, 1999, @(y,X) X\y, [0.05 0.95], clustid);
+%! bootclust ({X,y}, 1999, @mldivide, [0.05 0.95], clustid);
+%!
+%! % Please be patient, the calculations will be completed soon...
 
 %!demo
 %!
@@ -951,12 +981,12 @@ end
 %! y = randn (20,1); x = randn (20,1); X = [ones(20,1), x];
 %! stats = bootclust ({x,y}, 1999, @cor);
 %! stats = bootclust ({x,y}, 1999, @cor, [], clustid);
-%! stats = bootclust ({y,x}, 1999, @(y,x) pinv(x)*y); % Could use @regress
-%! stats = bootclust ({y,X}, 1999, @(y,X) pinv(X)*y);
-%! stats = bootclust ({y,X}, 1999, @(y,X) pinv(X)*y, [], clustid);
-%! stats = bootclust ({y,X}, 1999, @(y,X) pinv(X)*y, [], clustid, true);
-%! stats = bootclust ({y,X}, 1999, @(y,X) pinv(X)*y, [], clustid, true, 1);
-%! stats = bootclust ({y,X}, 1999, @(y,X) pinv(X)*y, [.05,.95], clustid);
+%! stats = bootclust ({x,y}, 1999, @mldivide);
+%! stats = bootclust ({X,y}, 1999, @mldivide);
+%! stats = bootclust ({X,y}, 1999, @mldivide, [], clustid);
+%! stats = bootclust ({X,y}, 1999, @mldivide, [], clustid, true);
+%! stats = bootclust ({X,y}, 1999, @mldivide, [], clustid, true, 1);
+%! stats = bootclust ({X,y}, 1999, @mldivide, [.05,.95], clustid);
 
 %!test
 %! % Air conditioning failure times in Table 1.2 of Davison A.C. and
@@ -974,10 +1004,10 @@ end
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
 %!   assert (stats.original, 108.0833333333333, 1e-08);
-%!   assert (stats.bias, -2.842170943040401e-14, 1e-08);
-%!   assert (stats.std_error, 38.21311346451331, 1e-08);
-%!   assert (stats.CI_lower, 37.63178001821489, 1e-08);
-%!   assert (stats.CI_upper, 200.846030428085, 1e-08);
+%!   assert (stats.bias, 8.526512829121202e-14, 1e-08);
+%!   assert (stats.std_error, 39.13480972156448, 1e-08);
+%!   assert (stats.CI_lower, 35.93016716402185, 1e-08);
+%!   assert (stats.CI_upper, 202.9345038615351, 1e-08);
 %! end
 %!
 %! % Nonparametric 95% expanded BCa confidence intervals
@@ -990,10 +1020,10 @@ end
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
 %!   assert (stats.original, 108.0833333333333, 1e-08);
-%!   assert (stats.bias, -2.842170943040401e-14, 1e-08);
-%!   assert (stats.std_error, 38.21311346451331, 1e-08);
-%!   assert (stats.CI_lower, 49.68734232019933, 1e-08);
-%!   assert (stats.CI_upper, 232.2854912518225, 1e-08);
+%!   assert (stats.bias, 8.526512829121202e-14, 1e-08);
+%!   assert (stats.std_error, 39.13480972156448, 1e-08);
+%!   assert (stats.CI_lower, 48.80198765188484, 1e-08);
+%!   assert (stats.CI_upper, 246.531254466907, 1e-08);
 %! end
 %!
 %! % Exact intervals based on an exponential model are 65.9 - 209.2
@@ -1013,10 +1043,10 @@ end
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
 %!   assert (stats.original, 171.534023668639, 1e-08);
-%!   assert (stats.bias, -7.305657266503118, 1e-08);
-%!   assert (stats.std_error, 43.17157379039285, 1e-08);
-%!   assert (stats.CI_lower, 95.33589724383222, 1e-08);
-%!   assert (stats.CI_upper, 237.1866652820803, 1e-08);
+%!   assert (stats.bias, -6.916841556872669, 1e-08);
+%!   assert (stats.std_error, 42.5668171689963, 1e-08);
+%!   assert (stats.CI_lower, 96.28173295410532, 1e-08);
+%!   assert (stats.CI_upper, 236.5630928698122, 1e-08);
 %! end
 %!
 %! % Nonparametric 90% BCa confidence intervals
@@ -1026,10 +1056,10 @@ end
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
 %!   assert (stats.original, 171.534023668639, 1e-08);
-%!   assert (stats.bias, -7.305657266503118, 1e-08);
-%!   assert (stats.std_error, 43.17157379039285, 1e-08);
-%!   assert (stats.CI_lower, 113.2858617321027, 1e-08);
-%!   assert (stats.CI_upper, 264.0328613673329, 1e-08);
+%!   assert (stats.bias, -6.916841556872669, 1e-08);
+%!   assert (stats.std_error, 42.5668171689963, 1e-08);
+%!   assert (stats.CI_lower, 117.0129329921189, 1e-08);
+%!   assert (stats.CI_upper, 269.5860065995567, 1e-08);
 %! end
 %!
 %! % Exact intervals based on normal theory are 118.4 - 305.2 (Table 14.2)
@@ -1052,10 +1082,10 @@ end
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
 %!   assert (stats.original, 0.7763744912894071, 1e-08);
-%!   assert (stats.bias, -0.007614791775856333, 1e-08);
-%!   assert (stats.std_error, 0.1355245146889644, 1e-08);
-%!   assert (stats.CI_lower, 0.5146251204001586, 1e-08);
-%!   assert (stats.CI_upper, 0.9531054945982934, 1e-08);
+%!   assert (stats.bias, -0.007472442917410338, 1e-08);
+%!   assert (stats.std_error, 0.1355235987517371, 1e-08);
+%!   assert (stats.CI_lower, 0.5147126713756366, 1e-08);
+%!   assert (stats.CI_upper, 0.953156157402217, 1e-08);
 %! end
 %!
 %! % Nonparametric 90% BCa confidence intervals
@@ -1065,8 +1095,8 @@ end
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
 %!   assert (stats.original, 0.7763744912894071, 1e-08);
-%!   assert (stats.bias, -0.007614791775856333, 1e-08);
-%!   assert (stats.std_error, 0.1355245146889644, 1e-08);
-%!   assert (stats.CI_lower, 0.4177828971982108, 1e-08);
-%!   assert (stats.CI_upper, 0.9238952404759969, 1e-08);
+%!   assert (stats.bias, -0.007472442917410338, 1e-08);
+%!   assert (stats.std_error, 0.1355235987517371, 1e-08);
+%!   assert (stats.CI_lower, 0.4179653612679617, 1e-08);
+%!   assert (stats.CI_upper, 0.9240227537437981, 1e-08);
 %! end
