@@ -37,11 +37,11 @@
 %     'bootknife (DATA, NBOOT, BOOTFUN)' also specifies BOOTFUN: the function
 %     calculated on the original sample and the bootstrap resamples. BOOTFUN
 %     must be either a:
-%       <> function handle or anonymous function,
-%       <> string of function name, or
+%       <> function handle, function name or an anonymous function,
+%       <> string of a function name, or
 %       <> a cell array where the first cell is one of the above function
 %          definitions and the remaining cells are (additional) input arguments 
-%          to that function (other than the data arguments).
+%          to that function (after the data arguments).
 %        In all cases BOOTFUN must take DATA for the initial input argument(s).
 %        BOOTFUN can return a scalar or any multidimensional numeric variable,
 %        but the output will be reshaped as a column vector. BOOTFUN must
@@ -106,12 +106,8 @@
 %     drawn from the nonscalar DATA argument to create that sample.
 %
 %  * For cluster resampling, use the 'bootclust' function instead. Clustered
-%    or serially dependent data can also be analysed by the 'bootwild' and
-%    'bootbayes' functions.
-%
-%  REQUIREMENTS:
-%    The function file boot.m (or better boot.mex) and bootcdf, which are
-%    distributed with the statistics-resampling package.
+%    or serially dependent data can also be analysed by the 'bootstrp', 
+%    'bootwild' and 'bootbayes' functions.
 %
 %  DETAILS:
 %    For a DATA sample with n rows, bootknife resampling involves creating
@@ -187,8 +183,8 @@ function [stats, bootstat, bootsam] = bootknife (x, nboot, bootfun, alpha, ...
   % REF, ISOCTAVE, ERRCHK and LOO are undocumented input arguments required for
   % some of the features of bootknife or the functions that require it
 
-  % Store local functions in a stucture for parallel processes
-  localfunc = struct ('col2args', @col2args, ...
+  % Store subfunctions in a stucture to make them available for parallel processes
+  parsubfun = struct ('col2args', @col2args, ...
                       'kdeinv', @kdeinv, ...
                       'ExpandProbs', @ExpandProbs);
 
@@ -243,7 +239,7 @@ function [stats, bootstat, bootsam] = bootknife (x, nboot, bootfun, alpha, ...
       % array to a matrix and redefine bootfun to parse multiple input arguments
       szx = cellfun (@(x) size (x, 2), x);
       x = [x{:}];
-      bootfun = @(x) localfunc.col2args (bootfun, x, szx);
+      bootfun = @(x) parsubfun.col2args (bootfun, x, szx);
     else
       szx = size (x, 2);
     end
@@ -425,8 +421,8 @@ function [stats, bootstat, bootsam] = bootknife (x, nboot, bootfun, alpha, ...
       if (ISOCTAVE)
         % OCTAVE Parallel Computing Package is not installed or loaded
         warning ('bootknife:parallel', ...
-                 cat (2, 'Parallel Computing Package not installed and/or', ...
-                         ' loaded. Falling back to serial processing.'))
+                 cat (2, 'Parallel package is not installed and/or loaded.', ...
+                         ' Falling back to serial processing.'))
       else
         % MATLAB Parallel Computing Toolbox is not installed or loaded
         warning ('bootknife:parallel', ...
@@ -693,11 +689,11 @@ function [stats, bootstat, bootsam] = bootknife (x, nboot, bootfun, alpha, ...
     % Bootstrap standard error
     se = std (bootstat, 0, 2);  % Unbiased since we used bootknife resampling
     if (~ isnan (alpha))
-      % If bootfun is the arithmetic meam, expand the probabilities of the 
+      % If bootfun is the arithmetic mean, expand the probabilities of the 
       % percentiles using Student's t-distribution
       if (strcmpi (bootfun_str, 'mean'))
         expan_alpha = (3 - nalpha) * ...
-                      localfunc.ExpandProbs (alpha / (3 - nalpha), n - K, LOO);
+                      parsubfun.ExpandProbs (alpha / (3 - nalpha), n - K, LOO);
       else
         expan_alpha = alpha;
       end
@@ -718,21 +714,38 @@ function [stats, bootstat, bootsam] = bootknife (x, nboot, bootfun, alpha, ...
           % Attempt to form bias-corrected and accelerated (BCa) bootstrap CIs
           % Use the Jackknife to calculate the acceleration constant (a)
           try
-            jackfun = @(i) bootfun (x(1 : n ~= i, :));
-            if (ncpus > 1)  
-              % PARALLEL evaluation of bootfun on each jackknife resample 
-              if (ISOCTAVE)
-                % OCTAVE
-                T = cell2mat (pararrayfun (ncpus, jackfun, 1 : n, ...
-                                           'UniformOutput', false));
+            if (vectorized)
+              % Leave-one-out DATA resampling followed by vectorized function
+              % evaluations
+              if (nvar > 1)
+                % Multivariate
+                T = bootfun (reshape (cell2mat (arrayfun (...
+                             @(i) x(1 : n ~= i, :), (1 : n)', ...
+                             'UniformOutput', false)), n - 1, []));
               else
-                % MATLAB
-                T = zeros (m, n);
-                parfor i = 1 : n; T(:, i) = feval (jackfun, i); end
+                % Univariate
+                T = bootfun (cell2mat (arrayfun (@(i) x(1 : n ~= i, :), ...
+                             (1 : n), 'UniformOutput', false)));
               end
             else
-              % SERIAL evaluation of bootfun on each jackknife resample
-              T = cell2mat (arrayfun (jackfun, 1 : n, 'UniformOutput', false));
+              % Leave-one-out DATA resampling followed by looped function
+              % evaluations (if bootfun is not vectorized)
+              jackfun = @(i) bootfun (x(1 : n ~= i, :));
+              if (ncpus > 1)  
+                % PARALLEL evaluation of bootfun on each jackknife resample 
+                if (ISOCTAVE)
+                  % OCTAVE
+                  T = cell2mat (pararrayfun (ncpus, jackfun, 1 : n, ...
+                                             'UniformOutput', false));
+                else
+                  % MATLAB
+                  T = zeros (m, n);
+                  parfor i = 1 : n; T(:, i) = feval (jackfun, i); end
+                end
+              else
+                % SERIAL evaluation of bootfun on each jackknife resample
+                T = cell2mat (arrayfun (jackfun, 1 : n, 'UniformOutput', false));
+              end
             end
             % Calculate empirical influence function
             if (~ isempty (strata))
@@ -771,12 +784,15 @@ function [stats, bootstat, bootsam] = bootknife (x, nboot, bootfun, alpha, ...
       ci = zeros (m, 2);
       for j = 1 : m
         try
-          ci(j, :) = localfunc.kdeinv (l(j, :), bootstat(j, :), ...
+          if (LOO)
+            ci(j, :) = parsubfun.kdeinv (l(j, :), bootstat(j, :), ...
                                    se(j) * sqrt (1 / (n - K)), 1 - 1 / (n - K));
+          else
+            error ('Invoke linear interpolation method.')
+          end
         catch
-          % Linear interpolation (legacy)
-          fprintf (strcat ('Note: Falling back to linear interpolation to', ...
-                           ' calculate percentiles for interval pair %u\n'), j);
+          % Linear interpolation (legacy) when LOO is false and for corner cases
+          % where KDE fails
           [t1, cdf] = bootcdf (bootstat(j, :), true, 1);
           ci(j, 1) = interp1 (cdf, t1, l(j, 1), 'linear', min (t1));
           ci(j, 2) = interp1 (cdf, t1, l(j, 2), 'linear', max (t1));
@@ -1048,6 +1064,8 @@ end
 %!
 %! % 95% expanded BCa bootstrap confidence intervals for the mean
 %! bootknife (data, 1999, @mean);
+%!
+%! % Please be patient, the calculations will be completed soon...
 
 %!demo
 %!
@@ -1081,6 +1099,8 @@ end
 %! % 90% equal-tailed percentile bootstrap confidence intervals for
 %! % the variance
 %! bootknife (data, 1999, {@var, 1}, 0.1);
+%!
+%! % Please be patient, the calculations will be completed soon...
 
 %!demo
 %!
@@ -1090,6 +1110,8 @@ end
 %!
 %! % 90% BCa bootstrap confidence intervals for the variance
 %! bootknife (data, 1999, {@var, 1}, [0.05 0.95]);
+%!
+%! % Please be patient, the calculations will be completed soon...
 
 %!demo
 %!
@@ -1120,7 +1142,12 @@ end
 %! y = randn (20,1); x = randn (20,1); X = [ones(20,1), x];
 %!
 %! % 90% BCa confidence interval for regression coefficients 
-%! bootknife ({y,X}, 1999, @(y,X) X\y, [0.05 0.95]); % Could also use @regress
+%! bootknife ({X,y}, 1999, @mldivide, [0.05 0.95]);
+%!
+%! % N.B: Consider using either the 'bootwild', 'bootbayes' or 'bootlm'
+%! % functions for bootstrapping linear regression problems instead
+%!
+%! % Please be patient, the calculations will be completed soon...
 
 %!demo
 %!
@@ -1226,6 +1253,8 @@ end
 %!   %<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 %! 
 %! end
+%!
+%! % Please be patient, the calculations will be completed soon...
 
 %!demo
 %!
@@ -1411,10 +1440,10 @@ end
 %! %
 %! % method                                |   0.05 |   0.95 | length | shape |
 %! % --------------------------------------|--------|--------|--------|-------|
-%! % ci2  - percentile (equal-tailed)      |   96.1 |  237.0 |  140.9 |  0.87 |
-%! % ci4  - BCa                            |  115.9 |  264.6 |  148.7 |  1.68 |
-%! % ci6a - calibrated (equal-tailed)      |   82.6 |  254.4 |  171.8 |  0.93 |
-%! % ci6b - calibrated                     |  113.4 |  284.9 |  171.5 |  1.95 |
+%! % ci2  - percentile (equal-tailed)      |   96.2 |  237.2 |  141.0 |  0.87 |
+%! % ci4  - BCa                            |  115.3 |  263.8 |  148.5 |  1.64 |
+%! % ci6a - calibrated (equal-tailed)      |   83.0 |  252.9 |  169.9 |  0.92 |
+%! % ci6b - calibrated                     |  113.2 |  282.4 |  169.2 |  1.90 |
 %! % --------------------------------------|--------|--------|--------|-------|
 %! % parametric - exact                    |  118.4 |  305.2 |  186.8 |  2.52 |
 %! %
@@ -1496,11 +1525,10 @@ end
 %!   y = randn (20,1); x = randn (20,1); X = [ones(20,1), x];
 %!   stats = bootknife ({x,y}, 1999, @cor);
 %!   stats = bootknife ({x,y}, 1999, @cor, [], strata);
-%!   stats = bootknife ({y,x}, 1999, @(y,x) pinv(x)*y); % Could use @regress
-%!   stats = bootknife ({y,X}, 1999, @(y,X) pinv(X)*y);
-%!   stats = bootknife ({y,X}, 1999, @(y,X) pinv(X)*y, [], strata);
-%!   stats = bootknife ({y,X}, 1999, @(y,X) pinv(X)*y, [], strata, 2);
-%!   stats = bootknife ({y,X}, 1999, @(y,X) pinv(X)*y, [.05,.95], strata);
+%!   stats = bootknife ({X,y}, 1999, @mldivide);
+%!   stats = bootknife ({X,y}, 1999, @mldivide, [], strata);
+%!   stats = bootknife ({X,y}, 1999, @mldivide, [], strata, 2);
+%!   stats = bootknife ({X,y}, 1999, @mldivide, [.05,.95], strata);
 %! catch
 %!   warning ('on', 'bootknife:parallel')
 %!   rethrow (lasterror)
@@ -1523,10 +1551,10 @@ end
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
 %!   assert (stats.original, 108.0833333333333, 1e-08);
-%!   assert (stats.bias, -2.842170943040401e-14, 1e-08);
-%!   assert (stats.std_error, 38.21311346451331, 1e-08);
-%!   assert (stats.CI_lower, 37.63178001821489, 1e-08);
-%!   assert (stats.CI_upper, 200.846030428085, 1e-08);
+%!   assert (stats.bias, 8.526512829121202e-14, 1e-08);
+%!   assert (stats.std_error, 39.13480972156448, 1e-08);
+%!   assert (stats.CI_lower, 35.93016716402185, 1e-08);
+%!   assert (stats.CI_upper, 202.9345038615352, 1e-08);
 %! end
 %!
 %! % Nonparametric 95% expanded BCa confidence intervals
@@ -1539,10 +1567,10 @@ end
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
 %!   assert (stats.original, 108.0833333333333, 1e-08);
-%!   assert (stats.bias, -2.842170943040401e-14, 1e-08);
-%!   assert (stats.std_error, 38.21311346451331, 1e-08);
-%!   assert (stats.CI_lower, 49.68734232019933, 1e-08);
-%!   assert (stats.CI_upper, 232.2854912518225, 1e-08);
+%!   assert (stats.bias, 8.526512829121202e-14, 1e-08);
+%!   assert (stats.std_error, 39.13480972156448, 1e-08);
+%!   assert (stats.CI_lower, 48.80198765188484, 1e-08);
+%!   assert (stats.CI_upper, 246.531254466907, 1e-08);
 %! end
 %!
 %! % Exact intervals based on an exponential model are 65.9 - 209.2
@@ -1562,10 +1590,10 @@ end
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
 %!   assert (stats.original, 171.534023668639, 1e-08);
-%!   assert (stats.bias, -7.305657266503118, 1e-08);
-%!   assert (stats.std_error, 43.17157379039285, 1e-08);
-%!   assert (stats.CI_lower, 95.33589724383222, 1e-08);
-%!   assert (stats.CI_upper, 237.1866652820803, 1e-08);
+%!   assert (stats.bias, -6.916841556872669, 1e-08);
+%!   assert (stats.std_error, 42.5668171689963, 1e-08);
+%!   assert (stats.CI_lower, 96.28173295410532, 1e-08);
+%!   assert (stats.CI_upper, 236.5630928698122, 1e-08);
 %! end
 %!
 %! % Nonparametric 90% BCa confidence intervals
@@ -1575,10 +1603,10 @@ end
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
 %!   assert (stats.original, 171.534023668639, 1e-08);
-%!   assert (stats.bias, -7.305657266503118, 1e-08);
-%!   assert (stats.std_error, 43.17157379039285, 1e-08);
-%!   assert (stats.CI_lower, 113.2858617321027, 1e-08);
-%!   assert (stats.CI_upper, 264.0328613673329, 1e-08);
+%!   assert (stats.bias, -6.916841556872669, 1e-08);
+%!   assert (stats.std_error, 42.5668171689963, 1e-08);
+%!   assert (stats.CI_lower, 117.0129329921189, 1e-08);
+%!   assert (stats.CI_upper, 269.5860065995567, 1e-08);
 %! end
 %!
 %! % Nonparametric 90% calibrated equal-tailed percentile confidence intervals
@@ -1587,10 +1615,10 @@ end
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
 %!   assert (stats.original, 171.534023668639, 1e-08);
-%!   assert (stats.bias, -8.09559197397769, 1e-08);
-%!   assert (stats.std_error, 46.23192443207778, 1e-08);
-%!   assert (stats.CI_lower, 81.57496114752357, 1e-08);
-%!   assert (stats.CI_upper, 255.5966823363611, 1e-08);
+%!   assert (stats.bias, -7.224310461851786, 1e-08);
+%!   assert (stats.std_error, 44.57463432863965, 1e-08);
+%!   assert (stats.CI_lower, 85.45679817042191, 1e-08);
+%!   assert (stats.CI_upper, 252.5744737190519, 1e-08);
 %! end
 %!
 %! % Nonparametric 90% calibrated percentile confidence intervals
@@ -1599,10 +1627,10 @@ end
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
 %!   assert (stats.original, 171.534023668639, 1e-08);
-%!   assert (stats.bias, -8.09559197397769, 1e-08);
-%!   assert (stats.std_error, 46.23192443207778, 1e-08);
-%!   assert (stats.CI_lower, 109.7569412835685, 1e-08);
-%!   assert (stats.CI_upper, 277.4147390170027, 1e-08);
+%!   assert (stats.bias, -7.224310461851786, 1e-08);
+%!   assert (stats.std_error, 44.57463432863965, 1e-08);
+%!   assert (stats.CI_lower, 114.620808678501, 1e-08);
+%!   assert (stats.CI_upper, 279.5543198945595, 1e-08);
 %! end
 %!
 %! % Exact intervals based on normal theory are 118.4 - 305.2 (Table 14.2)
@@ -1625,10 +1653,10 @@ end
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
 %!   assert (stats.original, 0.7763744912894071, 1e-08);
-%!   assert (stats.bias, -0.008329526543031429, 1e-08);
-%!   assert (stats.std_error, 0.1422804068655692, 1e-08);
-%!   assert (stats.CI_lower, 0.5046604076771914, 1e-08);
-%!   assert (stats.CI_upper, 0.9586661175519952, 1e-08);
+%!   assert (stats.bias, -0.008589727569214833, 1e-08);
+%!   assert (stats.std_error, 0.1440139406035633, 1e-08);
+%!   assert (stats.CI_lower, 0.5022123355180356, 1e-08);
+%!   assert (stats.CI_upper, 0.960132224676584, 1e-08);
 %! end
 %!
 %! % Nonparametric 90% BCa confidence intervals
@@ -1638,10 +1666,10 @@ end
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
 %!   assert (stats.original, 0.7763744912894071, 1e-08);
-%!   assert (stats.bias, -0.008329526543031429, 1e-08);
-%!   assert (stats.std_error, 0.1422804068655692, 1e-08);
-%!   assert (stats.CI_lower, 0.4094344122396682, 1e-08);
-%!   assert (stats.CI_upper, 0.9301552819706758, 1e-08);
+%!   assert (stats.bias, -0.008589727569214833, 1e-08);
+%!   assert (stats.std_error, 0.1440139406035633, 1e-08);
+%!   assert (stats.CI_lower, 0.3954734403669555, 1e-08);
+%!   assert (stats.CI_upper, 0.9323937461477884, 1e-08);
 %! end
 %!
 %! % Nonparametric 90% calibrated equal-tailed percentile confidence intervals
@@ -1650,10 +1678,10 @@ end
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
 %!   assert (stats.original, 0.7763744912894071, 1e-08);
-%!   assert (stats.bias, -0.009735795559474125, 1e-08);
-%!   assert (stats.std_error, 0.1446016217318865, 1e-08);
-%!   assert (stats.CI_lower, 0.3694847333563375, 1e-08);
-%!   assert (stats.CI_upper, 0.9783138564265896, 1e-08);
+%!   assert (stats.bias, -0.009947337525569089, 1e-08);
+%!   assert (stats.std_error, 0.1475068053421092, 1e-08);
+%!   assert (stats.CI_lower, 0.3299014421715357, 1e-08);
+%!   assert (stats.CI_upper, 0.9769208665154616, 1e-08);
 %! end
 %!
 %! % Nonparametric 90% calibrated percentile confidence intervals
@@ -1662,9 +1690,9 @@ end
 %! if (isempty (regexp (which ('boot'), 'mex$')))
 %!   % test boot m-file result
 %!   assert (stats.original, 0.7763744912894071, 1e-08);
-%!   assert (stats.bias, -0.009735795559474125, 1e-08);
-%!   assert (stats.std_error, 0.1446016217318865, 1e-08);
-%!   assert (stats.CI_lower, 0.2078720903008157, 1e-08);
-%!   assert (stats.CI_upper, 0.9419996961228243, 1e-08);
+%!   assert (stats.bias, -0.009947337525569089, 1e-08);
+%!   assert (stats.std_error, 0.1475068053421092, 1e-08);
+%!   assert (stats.CI_lower, 0.2237810002752923, 1e-08);
+%!   assert (stats.CI_upper, 0.945020625755707, 1e-08);
 %! end
 %! % Exact intervals based on normal theory are 0.51 - 0.91
